/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.poshi.runner.util;

import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

import java.util.Objects;

/**
 * @author Kevin Yen
 */
public class ExternalMethod {

	public static Object execute(
			Method method, Object object, Object[] parameters)
		throws Exception {

		Object returnObject = null;

		parameters = _transformParameters(parameters);

		try {
			returnObject = method.invoke(object, parameters);
		}
		catch (Exception e) {
			Throwable throwable = e.getCause();

			if (throwable != null) {
				throw new Exception(throwable.getMessage(), e);
			}
			else {
				throw e;
			}
		}

		if (returnObject == null) {
			return "";
		}

		return returnObject;
	}

	public static Object execute(
			String methodName, Object object, Object[] parameters)
		throws Exception {

		Class<?> clazz = object.getClass();

		Method method = getMethod(clazz, methodName, parameters);

		return execute(method, object, parameters);
	}

	public static Object execute(String className, String methodName)
		throws Exception {

		Object[] parameters = {};

		return execute(className, methodName, parameters);
	}

	public static Object execute(
			String className, String methodName, Object[] parameters)
		throws Exception {

		Class<?> clazz = Class.forName(className);

		Method method = getMethod(clazz, methodName, parameters);

		int modifiers = method.getModifiers();

		Object object = null;

		if (!Modifier.isStatic(modifiers)) {
			object = clazz.newInstance();
		}

		return execute(method, object, parameters);
	}

	public static Method getMethod(
		Class clazz, String methodName, Object[] parameters) {

		for (Method method : clazz.getMethods()) {
			if (!methodName.equals(method.getName())) {
				continue;
			}

			Class<?>[] methodParameterTypes = method.getParameterTypes();

			if (methodParameterTypes.length != parameters.length) {
				continue;
			}

			boolean parameterTypesMatch = true;

			for (int i = 0; i < methodParameterTypes.length; i++) {
				Object parameter = parameters[i];

				if (Objects.equals(parameter, _POSHI_NULL_NOTATION)) {
					continue;
				}

				if (!methodParameterTypes[i].isAssignableFrom(
						parameter.getClass())) {

					parameterTypesMatch = false;

					break;
				}
			}

			if (parameterTypesMatch) {
				return method;
			}
		}

		StringBuilder sb = new StringBuilder();

		sb.append("Unable to find method '");
		sb.append(methodName);
		sb.append("' of class '");
		sb.append(clazz.getCanonicalName());

		if ((parameters != null) && (parameters.length != 0)) {
			sb.append("' with parameters types: (");

			for (Object parameter : parameters) {
				Class<?> parameterType = parameter.getClass();

				sb.append(parameterType.toString());

				sb.append(", ");
			}

			sb.delete(sb.length() - 2, sb.length());

			sb.append(")");
		}

		throw new IllegalArgumentException(sb.toString());
	}

	private static Object[] _transformParameters(Object[] parameters) {
		for (int i = 0; i < parameters.length; i++) {
			if (Objects.equals(parameters[i], _POSHI_NULL_NOTATION)) {
				parameters[i] = null;
			}
		}

		return parameters;
	}

	private static final String _POSHI_NULL_NOTATION = "Poshi.NULL";

}